@@ -31,14 +31,12 @@ char *get_dbf_record(dbhead_t *dbh, long rec_num)
 	if (rec_num > dbh->db_records) {
 		return NULL;
 	}
-	if ((cp = (char *)malloc(dbh->db_rlen)) == NULL) {
-		return NULL;
-	}
+	cp = (char *)emalloc(dbh->db_rlen);
 
 	/* go to the correct spot on the file */
 	offset = dbh->db_hlen + (rec_num - 1) * dbh->db_rlen;
 	if (get_piece(dbh, offset, cp, dbh->db_rlen) != dbh->db_rlen) {
-		free(cp);
+		efree(cp);
 		cp = NULL;
 	}
 	if (cp)
@@ -90,6 +88,17 @@ int put_piece(dbhead_t *dbh, long offset, char *cp, int len)
 	return write(dbh->db_fd, cp, len);
 }
 
+int put_dbf_eof_marker(dbhead_t *dbh)
+{
+	char eof = 0x1A;
+
+	if (put_piece(dbh, dbh->db_hlen + dbh->db_records * dbh->db_rlen, &eof, sizeof(eof)) != sizeof(eof)) {
+		php_error_docref(NULL, E_WARNING, "unable to write end-of-file marker");
+		return -1;
+	}
+	return 0;
+}
+
 int del_dbf_record(dbhead_t *dbh, long rec_num)
 {
 	int ret = 0;
@@ -100,48 +109,53 @@ int del_dbf_record(dbhead_t *dbh, long rec_num)
 	if ((cp = get_dbf_record(dbh, rec_num))) {
 		*cp = DELETED_RECORD;
 		ret = put_dbf_record(dbh, rec_num, cp);
-		free(cp);
+		efree(cp);
 	}
 	return ret;
 }
 
-void pack_dbf(dbhead_t *dbh)
+int pack_dbf(dbhead_t *dbh)
 {
 	long	out_off, in_off;
 	int	rec_cnt, new_cnt;
 	char	*cp;
 
-	if ((cp = (char *)malloc(dbh->db_rlen)) == NULL) {
-		return;
-	}
+	cp = (char *)emalloc(dbh->db_rlen);
 	in_off = out_off = dbh->db_hlen;
 
 	new_cnt = 0;
 	rec_cnt = dbh->db_records;
 	while (rec_cnt > 0) {
-		if (get_piece(dbh, in_off, cp, dbh->db_rlen) < 0)
-			break;
+		if (get_piece(dbh, in_off, cp, dbh->db_rlen) != dbh->db_rlen) {
+			php_error_docref(NULL, E_WARNING, "unable to read from the file");
+			efree(cp);
+			return -1;
+		}
 
 		if (*cp != DELETED_RECORD) {
 			/* write the record into the file */
-			if (put_piece(dbh, out_off, cp, dbh->db_rlen) < 0)
-				break;
+			if (put_piece(dbh, out_off, cp, dbh->db_rlen) != dbh->db_rlen) {
+				php_error_docref(NULL, E_WARNING, "unable to write to the file");
+				efree(cp);
+				return -1;
+			}
 			out_off += dbh->db_rlen;
 			new_cnt++;
 		}
 		in_off += dbh->db_rlen;
 		rec_cnt--;
 	}
-	free(cp);
+	efree(cp);
 
 	/* Try to truncate the file to the right size. */
 	if (ftruncate(dbh->db_fd, out_off) != 0) {
-	    TSRMLS_FETCH();
-	    php_error_docref(NULL TSRMLS_CC, E_WARNING, "dbase_pack() couldn't truncate the file to the right size. Some deleted records may still be left in there");
+	    php_error_docref(NULL, E_WARNING, "dbase_pack() couldn't truncate the file to the right size. Some deleted records may still be left in there");
+		return -1;
 	}
 
-	if (rec_cnt == 0)
-		dbh->db_records = new_cnt;
+	dbh->db_records = new_cnt;
+
+	return 0;
 }
 
 /* routine to get a field from a record */
@@ -150,11 +164,19 @@ char *get_field_val(char *rp, dbfield_t *fldp, char *cp)
 	int flen = fldp->db_flen;
 
 	if ( !cp )
-		cp = (char *)malloc(flen + 1);
-	if ( cp ) {
-		strncpy(cp, &rp[fldp->db_foffset], flen);
-		cp[flen] = 0;
-	}
+		cp = (char *)emalloc(flen + 1);
+	strncpy(cp, &rp[fldp->db_foffset], flen);
+	cp[flen] = '\0';
+	return cp;
+}
+
+char *get_binary_field_val(char *rp, dbfield_t *fldp, char *cp)
+{
+	int flen = fldp->db_flen;
+
+	if ( !cp )
+		cp = (char *)emalloc(flen);
+	memcpy(cp, &rp[fldp->db_foffset], flen);
 	return cp;
 }
 
@@ -163,24 +185,6 @@ void put_field_val(char *rp, dbfield_t *fldp, char *cp)
 	strncpy(&rp[fldp->db_foffset], cp, fldp->db_flen);
 }
 
-/*
- * output a record
- */
-void out_rec(dbhead_t *dbh, dbfield_t *dbf, char *cp)
-{
-	dbfield_t       *cur_f;
-	int     nfields = dbh->db_nfields;
-	char    *fnp = (char *)malloc(dbh->db_rlen);
-
-	printf("%c", *cp);
-	for (cur_f = dbf; cur_f < &dbf[nfields] ; cur_f++) {
-		printf(" ");
-		printf(cur_f->db_format, get_field_val(cp, cur_f, fnp));
-	}
-	printf("\n");
-	free(fnp);
-}
-
 /* check for record validity */
 int is_valid_rec(char *cp)
 {
